@@ -15,7 +15,7 @@ from typing import (
     cast,
 )
 
-import attr
+import attrs
 from outcome import Value
 
 from .. import _core
@@ -242,22 +242,22 @@ WRITABLE_FLAGS = (
 # To avoid this, we have to coalesce all the operations on a single socket
 # into one, and when the set of waiters changes we have to throw away the old
 # operation and start a new one.
-@attr.s(slots=True, eq=False)
+@attrs.define(eq=False)
 class AFDWaiters:
-    read_task: _core.Task | None = attr.ib(default=None)
-    write_task: _core.Task | None = attr.ib(default=None)
-    current_op: AFDPollOp | None = attr.ib(default=None)
+    read_task: _core.Task | None = None
+    write_task: _core.Task | None = None
+    current_op: AFDPollOp | None = None
 
 
 # We also need to bundle up all the info for a single op into a standalone
 # object, because we need to keep all these objects alive until the operation
 # finishes, even if we're throwing it away.
-@attr.s(slots=True, eq=False, frozen=True)
+@attrs.frozen(eq=False)
 class AFDPollOp:
-    lpOverlapped: CData = attr.ib()
-    poll_info: Any = attr.ib()
-    waiters: AFDWaiters = attr.ib()
-    afd_group: AFDGroup = attr.ib()
+    lpOverlapped: CData
+    poll_info: Any
+    waiters: AFDWaiters
+    afd_group: AFDGroup
 
 
 # The Windows kernel has a weird issue when using AFD handles. If you have N
@@ -271,22 +271,22 @@ class AFDPollOp:
 MAX_AFD_GROUP_SIZE = 500  # at 1000, the cubic scaling is just starting to bite
 
 
-@attr.s(slots=True, eq=False)
+@attrs.define(eq=False)
 class AFDGroup:
-    size: int = attr.ib()
-    handle: Handle = attr.ib()
+    size: int
+    handle: Handle
 
 
 assert not TYPE_CHECKING or sys.platform == "win32"
 
 
-@attr.s(slots=True, eq=False, frozen=True)
+@attrs.frozen(eq=False)
 class _WindowsStatistics:
-    tasks_waiting_read: int = attr.ib()
-    tasks_waiting_write: int = attr.ib()
-    tasks_waiting_overlapped: int = attr.ib()
-    completion_key_monitors: int = attr.ib()
-    backend: Literal["windows"] = attr.ib(init=False, default="windows")
+    tasks_waiting_read: int
+    tasks_waiting_write: int
+    tasks_waiting_overlapped: int
+    completion_key_monitors: int
+    backend: Literal["windows"] = attrs.field(init=False, default="windows")
 
 
 # Maximum number of events to dequeue from the completion port on each pass
@@ -405,10 +405,10 @@ def _afd_helper_handle() -> Handle:
     return handle
 
 
-@attr.s(frozen=True)
+@attrs.frozen(slots=False)
 class CompletionKeyEventInfo:
-    lpOverlapped: CData = attr.ib()
-    dwNumberOfBytesTransferred: int = attr.ib()
+    lpOverlapped: CData
+    dwNumberOfBytesTransferred: int
 
 
 class WindowsIOManager:
@@ -728,14 +728,70 @@ class WindowsIOManager:
 
     @_public
     async def wait_readable(self, sock: _HasFileNo | int) -> None:
+        """Block until the kernel reports that the given object is readable.
+
+        On Unix systems, ``sock`` must either be an integer file descriptor,
+        or else an object with a ``.fileno()`` method which returns an
+        integer file descriptor. Any kind of file descriptor can be passed,
+        though the exact semantics will depend on your kernel. For example,
+        this probably won't do anything useful for on-disk files.
+
+        On Windows systems, ``sock`` must either be an integer ``SOCKET``
+        handle, or else an object with a ``.fileno()`` method which returns
+        an integer ``SOCKET`` handle. File descriptors aren't supported,
+        and neither are handles that refer to anything besides a
+        ``SOCKET``.
+
+        :raises trio.BusyResourceError:
+            if another task is already waiting for the given socket to
+            become readable.
+        :raises trio.ClosedResourceError:
+            if another task calls :func:`notify_closing` while this
+            function is still working.
+        """
         await self._afd_poll(sock, "read_task")
 
     @_public
     async def wait_writable(self, sock: _HasFileNo | int) -> None:
+        """Block until the kernel reports that the given object is writable.
+
+        See `wait_readable` for the definition of ``sock``.
+
+        :raises trio.BusyResourceError:
+            if another task is already waiting for the given socket to
+            become writable.
+        :raises trio.ClosedResourceError:
+            if another task calls :func:`notify_closing` while this
+            function is still working.
+        """
         await self._afd_poll(sock, "write_task")
 
     @_public
     def notify_closing(self, handle: Handle | int | _HasFileNo) -> None:
+        """Notify waiters of the given object that it will be closed.
+
+        Call this before closing a file descriptor (on Unix) or socket (on
+        Windows). This will cause any `wait_readable` or `wait_writable`
+        calls on the given object to immediately wake up and raise
+        `~trio.ClosedResourceError`.
+
+        This doesn't actually close the object – you still have to do that
+        yourself afterwards. Also, you want to be careful to make sure no
+        new tasks start waiting on the object in between when you call this
+        and when it's actually closed. So to close something properly, you
+        usually want to do these steps in order:
+
+        1. Explicitly mark the object as closed, so that any new attempts
+           to use it will abort before they start.
+        2. Call `notify_closing` to wake up any already-existing users.
+        3. Actually close the object.
+
+        It's also possible to do them in a different order if that's more
+        convenient, *but only if* you make sure not to have any checkpoints in
+        between the steps. This way they all happen in a single atomic
+        step, so other tasks won't be able to tell what order they happened
+        in anyway.
+        """
         handle = _get_base_socket(handle)
         waiters = self._afd_waiters.get(handle)
         if waiters is not None:
@@ -748,12 +804,22 @@ class WindowsIOManager:
 
     @_public
     def register_with_iocp(self, handle: int | CData) -> None:
+        """TODO: these are implemented, but are currently more of a sketch than
+        anything real. See `#26
+        <https://github.com/python-trio/trio/issues/26>`__ and `#52
+        <https://github.com/python-trio/trio/issues/52>`__.
+        """
         self._register_with_iocp(handle, CKeys.WAIT_OVERLAPPED)
 
     @_public
     async def wait_overlapped(
         self, handle_: int | CData, lpOverlapped: CData | int
     ) -> object:
+        """TODO: these are implemented, but are currently more of a sketch than
+        anything real. See `#26
+        <https://github.com/python-trio/trio/issues/26>`__ and `#52
+        <https://github.com/python-trio/trio/issues/52>`__.
+        """
         handle = _handle(handle_)
         if isinstance(lpOverlapped, int):
             lpOverlapped = ffi.cast("LPOVERLAPPED", lpOverlapped)
@@ -845,6 +911,11 @@ class WindowsIOManager:
     async def write_overlapped(
         self, handle: int | CData, data: Buffer, file_offset: int = 0
     ) -> int:
+        """TODO: these are implemented, but are currently more of a sketch than
+        anything real. See `#26
+        <https://github.com/python-trio/trio/issues/26>`__ and `#52
+        <https://github.com/python-trio/trio/issues/52>`__.
+        """
         with ffi.from_buffer(data) as cbuf:
 
             def submit_write(lpOverlapped: _Overlapped) -> None:
@@ -870,6 +941,11 @@ class WindowsIOManager:
     async def readinto_overlapped(
         self, handle: int | CData, buffer: Buffer, file_offset: int = 0
     ) -> int:
+        """TODO: these are implemented, but are currently more of a sketch than
+        anything real. See `#26
+        <https://github.com/python-trio/trio/issues/26>`__ and `#52
+        <https://github.com/python-trio/trio/issues/52>`__.
+        """
         with ffi.from_buffer(buffer, require_writable=True) as cbuf:
 
             def submit_read(lpOverlapped: _Overlapped) -> None:
@@ -895,12 +971,22 @@ class WindowsIOManager:
 
     @_public
     def current_iocp(self) -> int:
+        """TODO: these are implemented, but are currently more of a sketch than
+        anything real. See `#26
+        <https://github.com/python-trio/trio/issues/26>`__ and `#52
+        <https://github.com/python-trio/trio/issues/52>`__.
+        """
         assert self._iocp is not None
         return int(ffi.cast("uintptr_t", self._iocp))
 
     @contextmanager
     @_public
     def monitor_completion_key(self) -> Iterator[tuple[int, UnboundedQueue[object]]]:
+        """TODO: these are implemented, but are currently more of a sketch than
+        anything real. See `#26
+        <https://github.com/python-trio/trio/issues/26>`__ and `#52
+        <https://github.com/python-trio/trio/issues/52>`__.
+        """
         key = next(self._completion_key_counter)
         queue = _core.UnboundedQueue[object]()
         self._completion_key_queues[key] = queue
