@@ -1,26 +1,35 @@
-# Runvar implementations
-import attr
+from __future__ import annotations
 
+from typing import Generic, TypeVar, cast
+
+# Runvar implementations
+import attrs
+
+from .._util import NoPublicConstructor, final
 from . import _run
 
-from .._util import Final
+T = TypeVar("T")
 
 
-@attr.s(eq=False, hash=False, slots=True)
-class _RunVarToken:
-    _no_value = object()
+@final
+class _NoValue: ...
 
-    _var = attr.ib()
-    previous_value = attr.ib(default=_no_value)
-    redeemed = attr.ib(default=False, init=False)
+
+@final
+@attrs.define(eq=False, hash=False)
+class RunVarToken(Generic[T], metaclass=NoPublicConstructor):
+    _var: RunVar[T]
+    previous_value: T | type[_NoValue] = _NoValue
+    redeemed: bool = attrs.field(default=False, init=False)
 
     @classmethod
-    def empty(cls, var):
-        return cls(var)
+    def _empty(cls, var: RunVar[T]) -> RunVarToken[T]:
+        return cls._create(var)
 
 
-@attr.s(eq=False, hash=False, slots=True)
-class RunVar(metaclass=Final):
+@final
+@attrs.define(eq=False, hash=False, repr=False)
+class RunVar(Generic[T]):
     """The run-local variant of a context variable.
 
     :class:`RunVar` objects are similar to context variable objects,
@@ -29,27 +38,27 @@ class RunVar(metaclass=Final):
 
     """
 
-    _NO_DEFAULT = object()
-    _name = attr.ib()
-    _default = attr.ib(default=_NO_DEFAULT)
+    _name: str
+    _default: T | type[_NoValue] = _NoValue
 
-    def get(self, default=_NO_DEFAULT):
+    def get(self, default: T | type[_NoValue] = _NoValue) -> T:
         """Gets the value of this :class:`RunVar` for the current run call."""
         try:
-            return _run.GLOBAL_RUN_CONTEXT.runner._locals[self]
+            return cast(T, _run.GLOBAL_RUN_CONTEXT.runner._locals[self])
         except AttributeError:
             raise RuntimeError("Cannot be used outside of a run context") from None
         except KeyError:
             # contextvars consistency
-            if default is not self._NO_DEFAULT:
-                return default
+            # `type: ignore` awaiting https://github.com/python/mypy/issues/15553 to be fixed & released
+            if default is not _NoValue:
+                return default  # type: ignore[return-value]
 
-            if self._default is not self._NO_DEFAULT:
-                return self._default
+            if self._default is not _NoValue:
+                return self._default  # type: ignore[return-value]
 
             raise LookupError(self) from None
 
-    def set(self, value):
+    def set(self, value: T) -> RunVarToken[T]:
         """Sets the value of this :class:`RunVar` for this current run
         call.
 
@@ -57,16 +66,16 @@ class RunVar(metaclass=Final):
         try:
             old_value = self.get()
         except LookupError:
-            token = _RunVarToken.empty(self)
+            token = RunVarToken._empty(self)
         else:
-            token = _RunVarToken(self, old_value)
+            token = RunVarToken[T]._create(self, old_value)
 
         # This can't fail, because if we weren't in Trio context then the
         # get() above would have failed.
         _run.GLOBAL_RUN_CONTEXT.runner._locals[self] = value
         return token
 
-    def reset(self, token):
+    def reset(self, token: RunVarToken[T]) -> None:
         """Resets the value of this :class:`RunVar` to what it was
         previously specified by the token.
 
@@ -82,14 +91,14 @@ class RunVar(metaclass=Final):
 
         previous = token.previous_value
         try:
-            if previous is _RunVarToken._no_value:
+            if previous is _NoValue:
                 _run.GLOBAL_RUN_CONTEXT.runner._locals.pop(self)
             else:
                 _run.GLOBAL_RUN_CONTEXT.runner._locals[self] = previous
         except AttributeError:
-            raise RuntimeError("Cannot be used outside of a run context")
+            raise RuntimeError("Cannot be used outside of a run context") from None
 
         token.redeemed = True
 
-    def __repr__(self):
-        return "<RunVar name={!r}>".format(self._name)
+    def __repr__(self) -> str:
+        return f"<RunVar name={self._name!r}>"
