@@ -9,6 +9,7 @@ import time
 import weakref
 from functools import partial
 from typing import (
+    TYPE_CHECKING,
     AsyncGenerator,
     Awaitable,
     Callable,
@@ -22,8 +23,6 @@ from typing import (
 
 import pytest
 import sniffio
-from outcome import Outcome
-from pytest import MonkeyPatch
 
 from .. import (
     CancelScope,
@@ -37,17 +36,23 @@ from .. import (
     sleep_forever,
 )
 from .._core._tests.test_ki import ki_self
-from .._core._tests.tutil import buggy_pypy_asyncgens, slow
+from .._core._tests.tutil import slow
 from .._threads import (
+    active_thread_count,
     current_default_thread_limiter,
     from_thread_check_cancelled,
     from_thread_run,
     from_thread_run_sync,
     to_thread_run_sync,
+    wait_all_threads_completed,
 )
-from ..lowlevel import Task
 from ..testing import wait_all_tasks_blocked
 
+if TYPE_CHECKING:
+    from outcome import Outcome
+
+    from ..lowlevel import Task
+
 RecordType = List[Tuple[str, Union[threading.Thread, Type[BaseException]]]]
 T = TypeVar("T")
 
@@ -234,9 +239,17 @@ def _get_thread_name(ident: int | None = None) -> str | None:
 
     libpthread_path = ctypes.util.find_library("pthread")
     if not libpthread_path:
-        print(f"no pthread on {sys.platform})")
+        # musl includes pthread functions directly in libc.so
+        # (but note that find_library("c") does not work on musl,
+        #  see: https://github.com/python/cpython/issues/65821)
+        # so try that library instead
+        # if it doesn't exist, CDLL() will fail below
+        libpthread_path = "libc.so"
+    try:
+        libpthread = ctypes.CDLL(libpthread_path)
+    except Exception:
+        print(f"no pthread on {sys.platform}")
         return None
-    libpthread = ctypes.CDLL(libpthread_path)
 
     pthread_getname_np = getattr(libpthread, "pthread_getname_np", None)
 
@@ -323,7 +336,9 @@ async def test_run_in_worker_thread() -> None:
     def g() -> NoReturn:
         raise ValueError(threading.current_thread())
 
-    with pytest.raises(ValueError) as excinfo:
+    with pytest.raises(
+        ValueError, match=r"^<Thread\(Trio thread \d+, started daemon \d+\)>$"
+    ) as excinfo:
         await to_thread_run_sync(g)
     print(excinfo.value.args)
     assert excinfo.value.args[0] != trio_thread
@@ -390,7 +405,7 @@ async def test_run_in_worker_thread_cancellation() -> None:
 # handled gracefully. (Requires that the thread result machinery be prepared
 # for call_soon to raise RunFinishedError.)
 def test_run_in_worker_thread_abandoned(
-    capfd: pytest.CaptureFixture[str], monkeypatch: MonkeyPatch
+    capfd: pytest.CaptureFixture[str], monkeypatch: pytest.MonkeyPatch
 ) -> None:
     monkeypatch.setattr(_core._thread_cache, "IDLE_TIMEOUT", 0.01)
 
@@ -570,11 +585,11 @@ async def test_run_in_worker_thread_limiter_error() -> None:
 
         def release_on_behalf_of(self, borrower: Task) -> NoReturn:
             record.append("release")
-            raise ValueError
+            raise ValueError("release on behalf")
 
     bs = BadCapacityLimiter()
 
-    with pytest.raises(ValueError) as excinfo:
+    with pytest.raises(ValueError, match="^release on behalf$") as excinfo:
         await to_thread_run_sync(lambda: None, limiter=bs)  # type: ignore[call-overload]
     assert excinfo.value.__context__ is None
     assert record == ["acquire", "release"]
@@ -583,13 +598,15 @@ async def test_run_in_worker_thread_limiter_error() -> None:
     # If the original function raised an error, then the semaphore error
     # chains with it
     d: dict[str, object] = {}
-    with pytest.raises(ValueError) as excinfo:
+    with pytest.raises(ValueError, match="^release on behalf$") as excinfo:
         await to_thread_run_sync(lambda: d["x"], limiter=bs)  # type: ignore[call-overload]
     assert isinstance(excinfo.value.__context__, KeyError)
     assert record == ["acquire", "release"]
 
 
-async def test_run_in_worker_thread_fail_to_spawn(monkeypatch: MonkeyPatch) -> None:
+async def test_run_in_worker_thread_fail_to_spawn(
+    monkeypatch: pytest.MonkeyPatch,
+) -> None:
     # Test the unlikely but possible case where trying to spawn a thread fails
     def bad_start(self: object, *args: object) -> NoReturn:
         raise RuntimeError("the engines canna take it captain")
@@ -850,7 +867,6 @@ async def test_from_thread_inside_trio_thread() -> None:
         from_thread_run_sync(not_called, trio_token=trio_token)
 
 
-@pytest.mark.skipif(buggy_pypy_asyncgens, reason="pypy 7.2.0 is buggy")
 def test_from_thread_run_during_shutdown() -> None:
     save = []
     record = []
@@ -913,7 +929,7 @@ async def test_recursive_to_thread() -> None:
         nonlocal tid
         tid = threading.get_ident()
         # The nesting of wrapper functions loses the return value of threading.get_ident
-        return from_thread_run(to_thread_run_sync, threading.get_ident)  # type: ignore[return-value]
+        return from_thread_run(to_thread_run_sync, threading.get_ident)  # type: ignore[no-any-return]
 
     assert tid != await to_thread_run_sync(get_tid_then_reenter)
 
@@ -1081,10 +1097,16 @@ async def test_reentry_doesnt_deadlock() -> None:
 
 
 async def test_cancellable_and_abandon_raises() -> None:
-    with pytest.raises(ValueError):
+    with pytest.raises(
+        ValueError,
+        match=r"^Cannot set `cancellable` and `abandon_on_cancel` simultaneously\.$",
+    ):
         await to_thread_run_sync(bool, cancellable=True, abandon_on_cancel=False)  # type: ignore[call-overload]
 
-    with pytest.raises(ValueError):
+    with pytest.raises(
+        ValueError,
+        match=r"^Cannot set `cancellable` and `abandon_on_cancel` simultaneously\.$",
+    ):
         await to_thread_run_sync(bool, cancellable=True, abandon_on_cancel=True)  # type: ignore[call-overload]
 
 
@@ -1094,3 +1116,50 @@ async def test_cancellable_warns() -> None:
 
     with pytest.warns(TrioDeprecationWarning):
         await to_thread_run_sync(bool, cancellable=True)
+
+
+async def test_wait_all_threads_completed() -> None:
+    no_threads_left = False
+    e1 = Event()
+    e2 = Event()
+
+    e1_exited = Event()
+    e2_exited = Event()
+
+    async def wait_event(e: Event, e_exit: Event) -> None:
+        def thread() -> None:
+            from_thread_run(e.wait)
+
+        await to_thread_run_sync(thread)
+        e_exit.set()
+
+    async def wait_no_threads_left() -> None:
+        nonlocal no_threads_left
+        await wait_all_threads_completed()
+        no_threads_left = True
+
+    async with _core.open_nursery() as nursery:
+        nursery.start_soon(wait_event, e1, e1_exited)
+        nursery.start_soon(wait_event, e2, e2_exited)
+        await wait_all_tasks_blocked()
+        nursery.start_soon(wait_no_threads_left)
+        await wait_all_tasks_blocked()
+        assert not no_threads_left
+        assert active_thread_count() == 2
+
+        e1.set()
+        await e1_exited.wait()
+        await wait_all_tasks_blocked()
+        assert not no_threads_left
+        assert active_thread_count() == 1
+
+        e2.set()
+        await e2_exited.wait()
+        await wait_all_tasks_blocked()
+        assert no_threads_left
+        assert active_thread_count() == 0
+
+
+async def test_wait_all_threads_completed_no_threads() -> None:
+    await wait_all_threads_completed()
+    assert active_thread_count() == 0
